/* sysALib.s - Mizar mz8115 system dependent assembly language routines */

/* Copyright 1988, Mizar Digital Systems, Inc. */

/*
modification history
--------------------
01c,25jul88,gae  updated to 4.0.
01b,11may88,miz  bug fixes.
01a,10feb88,miz  written by modifying v01a of mz7120/sysALib.s.
*/

/*
DESCRIPTION
This module contains system-dependent routines which must be written
in assembly language.  They perform functions such as locking and
unlocking interrupts, trapping to the ROM monitor, etc.

The first routine in this module is the system start-up code.
It is the first code executed after booting.

This module MUST be the first specified on the "ld" command that is used
to build the system.  sysInit is the entry point for UniWorks.

INTERNAL: Many routines in this module "link" and "unlk" the "c" frame pointer
a6@ although they don't use it in any way!  This is only for the benefit of
the stacktrace facility to allow it to properly trace tasks executing within
these routines.

*/

#define ASMLANGUAGE
#include "UniWorks.h"
#include "sysLib.h"
#include "config.h"

	/* internals */

	.globl	_sysInit	/* start of system code */
	.globl  _sysKernelTrap
	.globl  _sysVwTrap
	.globl  _sysVwTrapRtn
	.globl	_sysBclrIntA
	.globl	_sysAbortIntA

	/* externals */

	.globl	_usrInit	/* system initialization routine */
	.globl	_sysToMonitor	/* exit to monitor */

	.text
	.even

/*******************************************************************************
*
* sysInit - start after boot
*
* This is the system start-up entry point for UniWorks in ram.
* It is the first code executed after booting.
* It disables the interrupts, sets up the stack,
* and jumps to the C routine usrInit in usrConfig.c.
*
* The initial stack is set to grow down from sysInit.
* Note that this initial stack is used only by usrInit,
* then is never used again.
* Memory for the stack needs to be accounted for when determining the load
* address of the system.
*
* THIS IS NOT A CALLABLE ROUTINE.

* sysInit ()		/* THIS IS NOT A CALLABLE ROUTINE */

/*
*/

_sysInit:
	movew	#0x3700,sr		/* disable interrupts, turn on M bit */
	movel	#_sysInit,a7		/* set stack to grow down from code */
	movel	#BOOT_WARM_AUTOBOOT,a7@- /* push start type arg = WARM_BOOT */
	jsr	_usrInit		/* never returns - starts up kernel */

/*******************************************************************************
*
* sysKernelTrap - trap to kernel function
*
* This routine traps to the kernel. It can only be called from assembly
* language.
* The trap number for the kernel is defined in config.h.
*
* NOMANUAL

* sysKernelTrap ()

*/
_sysKernelTrap:
	trap	#TRAP_KERNEL
	rts

/*******************************************************************************
*
* sysVwTrap - trap to UniWorks function
*
* This routine traps to UniWorks. It can only be called from assembly
* language.
* The trap number for UniWorks is defined in config.h.
*
* NOMANUAL

* sysVwTrap ()

*/
_sysVwTrap:
	trap	#TRAP_UNIWORKS
	rts

/*
 * This routine calls the routine whose address is in a0.  Since the system
 * is now in supervisor state (since we got here from a trap) this can be
 * used to call a routine in supervisor state.
 * THIS IS NOT A C CALLABLE ROUTINE!!
 */

_sysVwTrapRtn:
	jsr	a0@		/* vector to proper routine. */
	rte			/* return from trap */

/*******************************************************************************
*
* sysBclrIntA - Handle /BCLR interrupt 
*
* This routine releases the VME bus on /BCLR interrupt.  Even if you
* were not previously in ROR-enable, it forces you to that mode.
* Release-on-BCLR is really not suitable for this environment.
*
* RETURNS: NONE.

* VOID sysBclrIntA ()

*/

_sysBclrIntA:
	moveb	#ENABLE_VME_ROR,ENABLE_BR_ADRS
				    /* enable bus release mechanism */
	rte

/*******************************************************************************
*
* sysAbortIntA - Handle abort switch interrupt 
*
* This routine returns to UniWorks boot monitor on abort interrupt
*
* RETURNS: NONE.

* VOID sysAbortIntA ()

*/

_sysAbortIntA:
	jsr	_sysToMonitor		/* go to monitor land */
